##Procedure Header
# Name:
#    ixiangpf::emulation_pce_config
#
# Description:
#    This procedure will configure PCE
#
# Synopsis:
#    ixiangpf::emulation_pce_config
#        -mode                                         CHOICES create
#                                                      CHOICES delete
#                                                      CHOICES modify
#                                                      CHOICES enable
#                                                      CHOICES disable
#        [-port_handle                                 REGEXP ^[0-9]+/[0-9]+/[0-9]+$]
#        [-handle                                      ANY]
#x       [-pce_active                                  ANY]
#x       [-pc_reply_lsps_per_pcc                       NUMERIC]
#x       [-pcc_ipv4_address                            ANY]
#x       [-pcc_ipv4_address_step                       ANY]
#x       [-max_lsps_per_pc_initiate                    ANY]
#x       [-keepalive_interval                          ANY]
#x       [-dead_interval                               ANY]
#x       [-pce_initiated_lsps_per_pcc                  NUMERIC]
#x       [-sr_pce_capability                           ANY]
#x       [-rate_control                                ANY]
#x       [-burst_interval                              ANY]
#x       [-max_initiated_lsp_per_interval              ANY]
#x       [-pcc_group_active                            ANY]
#x       [-path_setup_type                             CHOICES sr rsvpte]
#x       [-include_end_points                          ANY]
#x       [-include_srp                                 ANY]
#x       [-pce_initiate_include_lsp                    ANY]
#x       [-include_ero                                 ANY]
#x       [-pce_initiate_include_metric                 ANY]
#x       [-pce_initiate_include_bandwidth              ANY]
#x       [-pce_initiate_include_lspa                   ANY]
#x       [-pce_initiate_ip_version                     CHOICES ipv4 ipv6]
#x       [-src_end_point_ipv4                          ANY]
#x       [-src_end_point_ipv4_step                     ANY]
#x       [-dest_end_point_ipv4                         ANY]
#x       [-dest_end_point_ipv4_step                    ANY]
#x       [-src_end_point_ipv6                          ANY]
#x       [-src_end_point_ipv6_step                     ANY]
#x       [-dest_end_point_ipv6                         ANY]
#x       [-dest_end_point_ipv6_step                    ANY]
#x       [-override_srp_id_number                      CHOICES 0 1]
#x       [-srp_id_number                               ANY]
#x       [-override_plsp_id                            CHOICES 0 1]
#x       [-pce_initiate_plsp_id                        ANY]
#x       [-pce_initiate_include_symbolic_path_name_tlv ANY]
#x       [-pce_initiate_symbolic_path_name             ANY]
#x       [-pce_initiate_number_of_ero_sub_objects      NUMERIC]
#x       [-pce_initiate_number_of_metric_sub_object    NUMERIC]
#x       [-pce_initiate_bandwidth                      ANY]
#x       [-pce_initiate_setup_priority                 ANY]
#x       [-pce_initiate_holding_priority               ANY]
#x       [-pce_initiate_local_protection               ANY]
#x       [-pce_initiate_include_any                    ANY]
#x       [-pce_initiate_include_all                    ANY]
#x       [-pce_initiate_exclude_any                    ANY]
#x       [-include_association                         ANY]
#x       [-association_id                              ANY]
#x       [-protection_lsp                              ANY]
#x       [-standby_mode                                ANY]
#x       [-pce_initiate_lsp_parameters_active          ANY]
#x       [-pcep_ero_sub_objects_list_active            ANY]
#x       [-loose_hop                                   ANY]
#x       [-sub_object_type                             CHOICES null
#x                                                     CHOICES ipv4prefix
#x                                                     CHOICES ipv6prefix
#x                                                     CHOICES asnumber]
#x       [-prefix_length                               ANY]
#x       [-ipv4_prefix                                 ANY]
#x       [-ipv4_prefix_step                            ANY]
#x       [-ipv6_prefix                                 ANY]
#x       [-ipv6_prefix_step                            ANY]
#x       [-as_number                                   ANY]
#x       [-sid_type                                    CHOICES null
#x                                                     CHOICES sid
#x                                                     CHOICES mplslabel20bit
#x                                                     CHOICES mplslabel32bit]
#x       [-sid                                         ANY]
#x       [-mpls_label                                  ANY]
#x       [-tc                                          ANY]
#x       [-bos                                         ANY]
#x       [-ttl                                         ANY]
#x       [-nai_type                                    CHOICES notapplicable
#x                                                     CHOICES ipv4nodeid
#x                                                     CHOICES ipv6nodeid
#x                                                     CHOICES ipv4adjacency
#x                                                     CHOICES ipv6adjacency
#x                                                     CHOICES unnumberedadjacencywithipv4nodeids]
#x       [-f_bit                                       ANY]
#x       [-ipv4_node_id                                ANY]
#x       [-ipv4_node_id_step                           ANY]
#x       [-ipv6_node_id                                ANY]
#x       [-ipv6_node_id_step                           ANY]
#x       [-local_ipv4_address                          ANY]
#x       [-local_ipv4_address_step                     ANY]
#x       [-remote_ipv4_address                         ANY]
#x       [-remote_ipv4_address_step                    ANY]
#x       [-local_ipv6_address                          ANY]
#x       [-local_ipv6_address_step                     ANY]
#x       [-remote_ipv6_address                         ANY]
#x       [-remote_ipv6_address_step                    ANY]
#x       [-local_node_id                               ANY]
#x       [-local_node_id_step                          ANY]
#x       [-local_interface_id                          ANY]
#x       [-remote_node_id                              ANY]
#x       [-remote_node_id_step                         ANY]
#x       [-remote_interface_id                         ANY]
#x       [-pcep_metric_sub_objects_list_active         ANY]
#x       [-metric_type                                 CHOICES igp tg hopcount msd]
#x       [-metric_value                                ANY]
#x       [-b_flag                                      ANY]
#x       [-response_options                            CHOICES crep crepwithnopath]
#x       [-response_path_type                          CHOICES sr rsvpte]
#x       [-include_rp                                  ANY]
#x       [-pc_reply_include_lsp                        ANY]
#x       [-enable_ero                                  ANY]
#x       [-pc_reply_include_metric                     ANY]
#x       [-pc_reply_include_bandwidth                  ANY]
#x       [-pc_reply_include_lspa                       ANY]
#x       [-reflect_r_p                                 ANY]
#x       [-request_id                                  ANY]
#x       [-enable_loose                                ANY]
#x       [-bi_directional                              ANY]
#x       [-priority_value                              ANY]
#x       [-pc_reply_number_of_ero_sub_objects          NUMERIC]
#x       [-pc_reply_number_of_metric_sub_object        NUMERIC]
#x       [-pc_reply_bandwidth                          ANY]
#x       [-reflect_l_s_p                               ANY]
#x       [-pc_reply_plsp_id                            ANY]
#x       [-pc_reply_include_symbolic_path_name_tlv     ANY]
#x       [-pc_reply_symbolic_path_name                 ANY]
#x       [-pc_reply_setup_priority                     ANY]
#x       [-pc_reply_holding_priority                   ANY]
#x       [-pc_reply_local_protection                   ANY]
#x       [-pc_reply_include_any                        ANY]
#x       [-pc_reply_include_all                        ANY]
#x       [-pc_reply_exclude_any                        ANY]
#x       [-nature_of_issue                             ANY]
#x       [-enable_c_flag                               ANY]
#x       [-reflected_object_no_path                    CHOICES elect_reflected_object
#x                                                     CHOICES metric
#x                                                     CHOICES bandwidth
#x                                                     CHOICES iro
#x                                                     CHOICES lspa]
#x       [-pc_reply_active                             ANY]
#x       [-pc_reply_name                               ANY]
#x       [-pc_request_ip_version                       CHOICES ipv4 ipv6]
#x       [-src_ipv4_address                            ANY]
#x       [-src_ipv4_address_step                       ANY]
#x       [-dest_ipv4_address                           ANY]
#x       [-dest_ipv4_address_step                      ANY]
#x       [-src_ipv6_address                            ANY]
#x       [-src_ipv6_address_step                       ANY]
#x       [-dest_ipv6_address                           ANY]
#x       [-dest_ipv6_address_step                      ANY]
#x       [-pc_request_active                           ANY]
#x       [-pc_request_name                             ANY]
#
# Arguments:
#    -mode
#    -port_handle
#    -handle
#        Specifies the parent node/object handle on which the pcep configuration should be configured.
#        In case modes – modify/delete/disable/enable – this denotes the object node handle on which the action needs to be performed. The handle value syntax is dependent on the vendor.
#x   -pce_active
#x       Activate/Deactivate Configuration
#x   -pc_reply_lsps_per_pcc
#x       Controls the maximum number of PCE LSPs that can be send as PATH Response.
#x   -pcc_ipv4_address
#x       IPv4 address of the PCC. This column is greyed out in case of PCEv6.
#x   -pcc_ipv4_address_step
#x   -max_lsps_per_pc_initiate
#x       Controls the maximum number of LSPs that can be present in a PCInitiate message.
#x   -keepalive_interval
#x       Frequency/Time Interval of sending PCEP messages to keep the session active.
#x   -dead_interval
#x       This is the time interval, after the expiration of which, a PCEP peer declares the session down if no PCEP message has been received.
#x   -pce_initiated_lsps_per_pcc
#x       Controls the maximum number of PCE LSPs that can be Initiated per PCC.
#x   -sr_pce_capability
#x       The SR PCE Capability TLV is an optional TLV associated with the OPEN Object to exchange SR capability of PCEP speakers.
#x   -rate_control
#x       The rate control is an optional feature associated with PCE initiated LSP.
#x   -burst_interval
#x       Interval in milisecond in which desired rate of messages needs to be maintained.
#x   -max_initiated_lsp_per_interval
#x       Maximum number of messages can be sent per interval.
#x   -pcc_group_active
#x       Activate/Deactivate Configuration
#x   -path_setup_type
#x       Indicates which type of LSP will be requested in the PCInitiated Request.
#x   -include_end_points
#x       Indicates whether END-POINTS object will be included in a PCInitiate message. All other attributes in sub-tab-End Points would be editable only if this checkbox is enabled
#x   -include_srp
#x       Indicates whether SRP object will be included in a PCInitiate message. All other attributes in sub-tab-SRP would be editable only if this checkbox is enabled.
#x   -pce_initiate_include_lsp
#x       Indicates whether LSP will be included in a PCInitiate message. All other attributes in sub-tab-LSP would be editable only if this checkbox is enabled.
#x   -include_ero
#x       Specifies whether ERO is active or inactive. All subsequent attributes of the sub-tab-ERO would be editable only if this is enabled.
#x   -pce_initiate_include_metric
#x       Indicates whether the PCInitiate message will have the metric list that is configured. All subsequent attributes of the sub-tab-Metric would be editable only if this is enabled.
#x   -pce_initiate_include_bandwidth
#x       Indicates whether Bandwidth will be included in a PCInitiate message. All other attributes in sub-tab-Bandwidth would be editable only if this checkbox is enabled.
#x   -pce_initiate_include_lspa
#x       Indicates whether LSPA will be included in a PCInitiate message. All other attributes in sub-tab-LSPA would be editable only if this checkbox is enabled.
#x   -pce_initiate_ip_version
#x       Drop down to select the IP Version with 2 choices : IPv4 / IPv6
#x   -src_end_point_ipv4
#x       Source IPv4 address of the path for which a path computation is Initiated. Will be greyed out if IP Version is set to IPv6.
#x   -src_end_point_ipv4_step
#x   -dest_end_point_ipv4
#x       Dest IPv4 address of the path for which a path computation is Initiated. Will be greyed out if IP Version is IPv6.
#x   -dest_end_point_ipv4_step
#x   -src_end_point_ipv6
#x       Source IPv6 address of the path for which a path computation is Initiated. Will be greyed out if IP version is set to IPv4.
#x   -src_end_point_ipv6_step
#x   -dest_end_point_ipv6
#x       Dest IPv6 address of the path for which a path computation is Initiated. Will be greyed out if IP Version is IPv4.
#x   -dest_end_point_ipv6_step
#x   -override_srp_id_number
#x       Indicates whether SRP ID Number is overridable.
#x   -srp_id_number
#x       The SRP object is used to correlate between initiation requests sent by the PCE and the error reports and state reports sent by the PCC. This number is unique per PCEP session and is incremented per initiation.
#x   -override_plsp_id
#x       Indicates if PLSP-ID will be set by the state machine or user. If disabled user wont have the control and state machine will set it.
#x   -pce_initiate_plsp_id
#x       An identifier for the LSP. A PCC creates a unique PLSP-ID for each LSP that is constant for the lifetime of a PCEP session. The PCC will advertise the same PLSP-ID on all PCEP sessions it maintains at a given time.
#x   -pce_initiate_include_symbolic_path_name_tlv
#x       Indicates if Symbolic-Path-Name TLV is to be included in PCInitiate message.
#x   -pce_initiate_symbolic_path_name
#x       Each LSP (path) must have a symbolic name that is unique in the PCC. It must remain constant throughout a path's lifetime, which may span across multiple consecutive PCEP sessions and/or PCC restarts.
#x   -pce_initiate_number_of_ero_sub_objects
#x       Value that indicates the number of ERO Sub Objects to be configured.
#x   -pce_initiate_number_of_metric_sub_object
#x       Value that indicates the number of Metric Objects to be configured.
#x   -pce_initiate_bandwidth
#x       Bandwidth (bits/sec)
#x   -pce_initiate_setup_priority
#x       The priority of the LSP with respect to taking resources.The value 0 is the highest priority.The Setup Priority is used in deciding whether this session can preempt another session.
#x   -pce_initiate_holding_priority
#x       The priority of the LSP with respect to holding resources. The value 0 is the highest priority.Holding Priority is used in deciding whether this session can be preempted by another session.
#x   -pce_initiate_local_protection
#x       When set, this means that the path must include links protected with Fast Reroute
#x   -pce_initiate_include_any
#x       This is a type of Resource Affinity Procedure that is used to validate a link. This control accepts a link if the link carries any of the attributes in the set.
#x   -pce_initiate_include_all
#x       This is a type of Resource Affinity Procedure that is used to validate a link. This control excludes a link from consideration if the link carries any of the attributes in the set.
#x   -pce_initiate_exclude_any
#x       This is a type of Resource Affinity Procedure that is used to validate a link. This control accepts a link only if the link carries all of the attributes in the set.
#x   -include_association
#x       Indicates whether PPAG will be included in a PCInitiate message. All other attributes in sub-tab-PPAG would be editable only if this checkbox is enabled.
#x   -association_id
#x       The Association ID of this LSP.
#x   -protection_lsp
#x       Indicates whether Protection LSP Bit is On.
#x   -standby_mode
#x       Indicates whether Standby LSP Bit is On.
#x   -pce_initiate_lsp_parameters_active
#x       Activate/Deactivate Configuration
#x   -pcep_ero_sub_objects_list_active
#x       Controls whether the ERO sub-object will be sent in the PCInitiate message.
#x   -loose_hop
#x       Indicates if user wants to represent a loose-hop sub object in the LSP
#x   -sub_object_type
#x       Using the Sub Object Type control user can configure which sub object needs to be included from the following options:
#x       Not Applicable
#x       IPv4 Prefix
#x       IPv6 Prefix
#x       AS Number.
#x   -prefix_length
#x       Prefix Length
#x   -ipv4_prefix
#x       IPv4 Prefix is specified as an IPv4 address.
#x   -ipv4_prefix_step
#x   -ipv6_prefix
#x       IPv6 Prefix is specified as an IPv6 address.
#x   -ipv6_prefix_step
#x   -as_number
#x       AS Number
#x   -sid_type
#x       Using the Segment Identifier Type control user can configure whether to include SID or not and if included what is its type. Types are as follows:
#x       Null
#x       SID
#x       20bit MPLS Label
#x       32bit MPLS Label.
#x       If it is Null then S bit is set in the packet. Default value is 20bit MPLS Label.
#x   -sid
#x       SIDis the Segment Identifier
#x   -mpls_label
#x       This control will be editable if the SID Type is set to either 20bit or 32bit MPLS-Label. This field will take the 20bit value of the MPLS-Label
#x   -tc
#x       This field is used to carry traffic class information. This control will be editable only if SID Type is MPLS Label 32bit.
#x   -bos
#x       This bit is set to true for the last entry in the label stack i.e., for the bottom of the stack, and false for all other label stack entries.
#x       This control will be editable only if SID Type is MPLS Label 32bit.
#x   -ttl
#x       This field is used to encode a time-to-live value. This control will be editable only if SID Type is MPLS Label 32bit.
#x   -nai_type
#x       NAI (Node or Adjacency Identifier) contains the NAI associated with the SID. Depending on the value of SID Type, the NAI can have different formats such as,
#x       Not Applicable
#x       IPv4 Node ID
#x       IPv6 Node ID
#x       IPv4 Adjacency
#x       IPv6 Adjacency
#x       Unnumbered Adjacency with IPv4 NodeIDs
#x   -f_bit
#x       A Flag which is used to carry additional information pertaining to SID. When this bit is set, the NAI value in the subobject body is null.
#x   -ipv4_node_id
#x       IPv4 Node ID is specified as an IPv4 address. This control can be configured if NAI Type is set to IPv4 Node ID and F bit is disabled.
#x   -ipv4_node_id_step
#x   -ipv6_node_id
#x       IPv6 Node ID is specified as an IPv6 address. This control can be configured if NAI Type is set to IPv6 Node ID and F bit is disabled.
#x   -ipv6_node_id_step
#x   -local_ipv4_address
#x       This Control can be configured if NAI Type is set to IPv4 Adjacency and F bit is disabled.
#x   -local_ipv4_address_step
#x   -remote_ipv4_address
#x       This Control can be configured if NAI Type is set to IPv4 Adjacency and F bit is disabled.
#x   -remote_ipv4_address_step
#x   -local_ipv6_address
#x       This Control can be configured if NAI Type is set to IPv6 Adjacency and F bit is disabled.
#x   -local_ipv6_address_step
#x   -remote_ipv6_address
#x       This Control can be configured if NAI Type is set to IPv6 Adjacency and F bit is disabled.
#x   -remote_ipv6_address_step
#x   -local_node_id
#x       This is the Local Node ID of the Unnumbered Adjacency with IPv4 NodeIDs which is specified as a pair of Node ID / Interface ID tuples.
#x       This Control can be configured if NAI Type is set to Unnumbered Adjacency with IPv4 NodeIDs and F bit is disabled.
#x   -local_node_id_step
#x   -local_interface_id
#x       This is the Local Interface ID of the Unnumbered Adjacency with IPv4 NodeIDs which is specified as a pair of Node ID / Interface ID tuples.
#x       This Control can be configured if NAI Type is set to Unnumbered Adjacency with IPv4 NodeIDs and F bit is disabled.
#x   -remote_node_id
#x       This is the Remote Node ID of the Unnumbered Adjacency with IPv4 NodeIDs which is specified as a pair of Node ID / Interface ID tuples.
#x       This Control can be configured if NAI Type is set to Unnumbered Adjacency with IPv4 NodeIDs and F bit is disabled.
#x   -remote_node_id_step
#x   -remote_interface_id
#x       This is the Remote Interface ID of the Unnumbered Adjacency with IPv4 NodeIDs which is specified as a pair of Node ID / Interface ID tuples.
#x       This Control can be configured if NAI Type is set to Unnumbered Adjacency with IPv4 NodeIDs and F bit is disabled.
#x   -pcep_metric_sub_objects_list_active
#x       Specifies whether the corresponding metric object is active or not.
#x   -metric_type
#x       This is a drop down which has 4 choices:IGP/ TE/ Hop count/ MSD.
#x   -metric_value
#x       User can specify the metric value corresponding to the metric type selected.
#x   -b_flag
#x       B (bound) flag MUST be set in the METRIC object, which specifies that the SID depth for the computed path MUST NOT exceed the metric-value.
#x   -response_options
#x       Reply Options
#x   -response_path_type
#x       Indicates which type of LSP will be responsed in the Path Request Response.
#x   -include_rp
#x       Include RP
#x   -pc_reply_include_lsp
#x       Include LSP
#x   -enable_ero
#x       Include ERO
#x   -pc_reply_include_metric
#x       Include Metric
#x   -pc_reply_include_bandwidth
#x       Include Bandwidth
#x   -pc_reply_include_lspa
#x       Include LSPA
#x   -reflect_r_p
#x       Reflect RP
#x   -request_id
#x       Request ID
#x   -enable_loose
#x       Loose
#x   -bi_directional
#x       Bi-directional
#x   -priority_value
#x       Priority
#x   -pc_reply_number_of_ero_sub_objects
#x       Number of ERO Sub Objects
#x   -pc_reply_number_of_metric_sub_object
#x       Number of Metric
#x   -pc_reply_bandwidth
#x       Bandwidth (bits/sec)
#x   -reflect_l_s_p
#x       Reflect LSP
#x   -pc_reply_plsp_id
#x       An identifier for the LSP. A PCC creates a unique PLSP-ID for each LSP that is constant for the lifetime of a PCEP session. The PCC will advertise the same PLSP-ID on all PCEP sessions it maintains at a given time.
#x   -pc_reply_include_symbolic_path_name_tlv
#x       Indicates if Symbolic-Path-Name TLV is to be included in PCInitiate message.
#x   -pc_reply_symbolic_path_name
#x       Each LSP (path) must have a symbolic name that is unique in the PCC. It must remain constant throughout a path's lifetime, which may span across multiple consecutive PCEP sessions and/or PCC restarts.
#x   -pc_reply_setup_priority
#x       Setup Priority
#x   -pc_reply_holding_priority
#x       Holding Priority
#x   -pc_reply_local_protection
#x       Local Protection
#x   -pc_reply_include_any
#x       Include Any
#x   -pc_reply_include_all
#x       Include All
#x   -pc_reply_exclude_any
#x       Exclude Any
#x   -nature_of_issue
#x       Nature Of Issue
#x   -enable_c_flag
#x       C Flag
#x   -reflected_object_no_path
#x       Reflected Object
#x   -pc_reply_active
#x       Activate/Deactivate Configuration
#x   -pc_reply_name
#x       Name of NGPF element, guaranteed to be unique in Scenario
#x   -pc_request_ip_version
#x       IP Version
#x   -src_ipv4_address
#x       Source IPv4 Address
#x   -src_ipv4_address_step
#x   -dest_ipv4_address
#x       Destination IPv4 Address
#x   -dest_ipv4_address_step
#x   -src_ipv6_address
#x       Source IPv6 Address
#x   -src_ipv6_address_step
#x   -dest_ipv6_address
#x       Destination IPv6 Address
#x   -dest_ipv6_address_step
#x   -pc_request_active
#x       Activate/Deactivate Configuration
#x   -pc_request_name
#x       Name of NGPF element, guaranteed to be unique in Scenario
#
# Return Values:
#    A list containing the ipv4 loopback protocol stack handles that were added by the command (if any).
#x   key:ipv4_loopback_handle                value:A list containing the ipv4 loopback protocol stack handles that were added by the command (if any).
#    A list containing the ipv4 protocol stack handles that were added by the command (if any).
#x   key:ipv4_handle                         value:A list containing the ipv4 protocol stack handles that were added by the command (if any).
#    $::SUCCESS | $::FAILURE
#    key:status                              value:$::SUCCESS | $::FAILURE
#    When status is $::FAILURE, contains more information
#    key:log                                 value:When status is $::FAILURE, contains more information
#    Handle of PCE configured
#    key:pce_handle                          value:Handle of PCE configured
#    Handle of PccGroup configured
#    key:pcc_group_handle                    value:Handle of PccGroup configured
#    Handle of PCE Initiated LSP Parameters configured
#    key:pce_initiate_lsp_parameters_handle  value:Handle of PCE Initiated LSP Parameters configured
#    Handle of PCReply LSP Parameters configured
#    key:pc_reply_lsp_parameters_handle      value:Handle of PCReply LSP Parameters configured
#    Handle of PCRequest Match Criteria configured
#    key:pc_request_match_criteria_handle    value:Handle of PCRequest Match Criteria configured
#
# Examples:
#
# Sample Input:
#
# Sample Output:
#
# Notes:
#
# See Also:
#

package ixiangpf;

use utils;
use ixiahlt;

sub emulation_pce_config {

	my $args = shift(@_);

	my @notImplementedParams = ();
	my @mandatoryParams = ();
	my @fileParams = ();

	# ixiahlt::logHltapiCommand('emulation_pce_config', $args);
	# ixiahlt::utrackerLog ('emulation_pce_config', $args);

	return ixiangpf::runExecuteCommand('emulation_pce_config', \@notImplementedParams, \@mandatoryParams, \@fileParams, $args);
}

# Return value for the package
return 1;
