# -*- coding: utf-8 -*-

import sys
from ixiaerror import IxiaError
from ixiangpf import IxiaNgpf
from ixiautil import PartialClass, make_hltapi_fail

class IxiaNgpf(PartialClass, IxiaNgpf):
	def traffic_handle_translator(self, **kwargs):
		r'''
		#Procedure Header
		 Name:
		    traffic_handle_translator
		
		 Description:
		    This is an internal method that is used translate handles generated by the
		    HLAPI framework into actual SDM IDs as well as any necessary additional
		    argument values.
		
		 Synopsis:
		    traffic_handle_translator
		x       [-emulation_src_handle ANY]
		x       [-emulation_dst_handle ANY]
		x       [-type                 CHOICES ethernet_vlan
		x                              CHOICES ipv4
		x                              CHOICES ipv6
		x                              CHOICES raw
		x                              CHOICES atm
		x                              CHOICES fcoe
		x                              CHOICES fc
		x                              CHOICES frame_relay
		x                              CHOICES hdlc
		x                              CHOICES ppp
		x                              CHOICES ipv4_application_traffic
		x                              CHOICES ipv6_application_traffic
		x                              CHOICES multicast_igmp
		x                              CHOICES avb1722
		x                              CHOICES avb_raw
		x                              DEFAULT ipv4]
		
		 Arguments:
		x   -emulation_src_handle
		x       A list of handles that need to act as sources for a traffic endpoint.
		x   -emulation_dst_handle
		x       A list of handles that need to act as destinations for a traffic endpoint.
		x   -type
		x       The type of traffic that we want to configure on the endpoint.
		
		 Return Values:
		    $::SUCCESS | $::FAILURE
		    key:status           value:$::SUCCESS | $::FAILURE
		    On status of failure, gives detailed information.
		    key:log              value:On status of failure, gives detailed information.
		    A list containing the names of all the arguments that were generated by
		    the current handle translation.
		    Please note that this list will be empty if no translation was performed.
		    key:translated_keys  value:
        A list containing the names of all the arguments that were generated by
        the current handle translation.
        Please note that this list will be empty if no translation was performed.
      
		    A list containing all the handles which were recognized by the framework
		    as being NGPF handles but which could not be mapped to any valid traffic
		    sources of the specified type.
		    key:invalid_handles  value:
        A list containing all the handles which were recognized by the framework
        as being NGPF handles but which could not be mapped to any valid traffic
        sources of the specified type.
      
		
		 Examples:
		
		 Sample Input:
		
		 Sample Output:
		
		 Notes:
		
		 See Also:
		
		'''
		hlpy_args = locals().copy()
		hlpy_args.update(kwargs)
		del hlpy_args['self']
		del hlpy_args['kwargs']

		not_implemented_params = []
		mandatory_params = []
		file_params = []

		try:
			return self.__execute_command(
				'traffic_handle_translator', 
				not_implemented_params, mandatory_params, file_params, 
				hlpy_args
			)
		except (IxiaError, ):
			e = sys.exc_info()[1]
			return make_hltapi_fail(e.message)
