##Library Header
# $Id: $
# Copyright  2003-2009 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_ospf_api.tcl
#
# Purpose:
#    A script development library containing OSPF APIs for test automation with
#    the Ixia chassis.
#
# Usage:
#    package require Ixia
#
# Description:
#    The procedures contained within this library include:
#
#    - emulation_ospf_config
#    - emulation_ospf_topology_route_config
#    - emulation_ospf_control
#    - emulation_ospf_lsa_config
#
# Requirements:
#    utils_ospf.tcl, a library containing ospf specific tcl utilities
#    ixiaapiutils.tcl, a library containing tcl utilities
#    parseddashedargs.tcl, a library containing the parse_dashed_args procedure
#
# Variables:
#
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################

proc ::ixia::emulation_ospf_config { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ospf_config $args\}]

        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable new_ixnetwork_api

    ::ixia::utrackerLog $procName $args

    keylset returnList status $::SUCCESS

    # Arguments
    set man_args {
        -port_handle  REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -mode         CHOICES create delete modify enable disable
        -session_type CHOICES ospfv2 ospfv3
                      DEFAULT ospfv2
    }

    set ixnetwork_man_args {
        -mode         CHOICES create delete modify enable disable
        -session_type CHOICES ospfv2 ospfv3
                      DEFAULT ospfv2
    }

     set opt_args {
        -area_id                    IP
                                    DEFAULT 0.0.0.0
        -area_id_step               IP
        -area_type                  CHOICES external-capable ppp stub
        -authentication_mode        CHOICES null simple md5
        -count                      RANGE   1-2000
                                    DEFAULT 1
        -dead_interval              RANGE   1-65535
        -demand_circuit             CHOICES 0 1
                                    DEFAULT 0
        -enable_support_rfc_5838    CHOICES 0 1
                                    DEFAULT 0
        -graceful_restart_enable    CHOICES 0 1
        -handle
        -hello_interval             RANGE   1-65535
        -ignore_db_desc_mtu         CHOICES 0 1
                                    DEFAULT 0
        -interface_cost             RANGE   0-4294967295
        -intf_ip_addr               IP
        -intf_ip_addr_step          IP
        -intf_prefix_length         RANGE   1-128
        -interface_handle
        -instance_id                RANGE   0-255
                                    DEFAULT 0
        -instance_id_step           RANGE   0-255
                                    DEFAULT 0
        -loopback_ip_addr           IP
                                    DEFAULT 0.0.0.0
        -loopback_ip_addr_step      IP
                                    DEFAULT 0.0.0.0
        -lsa_discard_mode           CHOICES 0 1
        -mac_address_init           MAC
        -mac_address_step           MAC
                                    DEFAULT 0000.0000.0001
        -md5_key
        -md5_key_id                 RANGE   0-255
        -mtu                        NUMERIC
        -network_type               CHOICES broadcast ptomp ptop
        -neighbor_intf_ip_addr      IP
        -neighbor_intf_ip_addr_step IP
        -neighbor_router_id         IPV4
        -neighbor_router_id_step    IPV4
                                    DEFAULT 0.0.1.0
        -option_bits                HEX
        -override_existence_check   CHOICES 0 1
                                    DEFAULT 0
        -override_tracking          CHOICES 0 1
                                    DEFAULT 0
        -password
        -reset
        -router_id                  IPV4
        -router_id_step             IPV4
                                    DEFAULT 0.0.1.0
        -router_priority            RANGE   0-255
        -te_enable                  CHOICES 0 1
        -te_max_bw                  REGEXP  ^[0-9]+
        -te_max_resv_bw             REGEXP  ^[0-9]+$
        -te_unresv_bw_priority0     REGEXP  ^[0-9]+$
        -te_unresv_bw_priority1     REGEXP  ^[0-9]+$
        -te_unresv_bw_priority2     REGEXP  ^[0-9]+$
        -te_unresv_bw_priority3     REGEXP  ^[0-9]+$
        -te_unresv_bw_priority4     REGEXP  ^[0-9]+$
        -te_unresv_bw_priority5     REGEXP  ^[0-9]+$
        -te_unresv_bw_priority6     REGEXP  ^[0-9]+$
        -te_unresv_bw_priority7     REGEXP  ^[0-9]+$
        -te_metric                  RANGE   1-2147483647
        -te_router_id               IPV4
        -vlan                       CHOICES 0 1
        -vlan_id_mode               CHOICES fixed increment
                                    DEFAULT increment
        -vlan_id                    RANGE   0-4096
        -vlan_id_step               RANGE   0-4096
                                    DEFAULT 1
        -vlan_user_priority         RANGE   0-7
                                    DEFAULT 0
        -atm_encapsulation          CHOICES VccMuxIPV4Routed
                                    CHOICES VccMuxIPV6Routed
                                    CHOICES VccMuxBridgedEthernetFCS
                                    CHOICES VccMuxBridgedEthernetNoFCS
                                    CHOICES LLCRoutedCLIP
                                    CHOICES LLCBridgedEthernetFCS
                                    CHOICES LLCBridgedEthernetNoFCS
        -bfd_registration           CHOICES 0 1
                                    DEFAULT 0
        -enable_dr_bdr              CHOICES 0 1
        -vci                        RANGE   0-65535
                                    DEFAULT 10
        -vci_step                   DEFAULT 1
        -vpi                        RANGE   0-255
                                    DEFAULT 32
        -vpi_step                   DEFAULT 1
        -get_next_session_mode      CHOICES from_array from_server
                                    DEFAULT from_array
        -no_write                   FLAG
        -te_admin_group             REGEXP ^[0-9]{2}([:.]{1}[0-9]{2}){3}$
        -validate_received_mtu      CHOICES 0 1 
                                    DEFAULT 1
        -graceful_restart_helper_mode_enable CHOICES 0 1 
                                    DEFAULT 0
        -strict_lsa_checking        CHOICES 0 1
                                    DEFAULT 1
        -support_reason_sw_restart  CHOICES 0 1
                                    DEFAULT 1
        -support_reason_sw_reload_or_upgrade    CHOICES 0 1
                                    DEFAULT 1
        -support_reason_switch_to_redundant_processor_control   CHOICES 0 1
                                    DEFAULT 1
        -support_reason_unknown     CHOICES 0 1
                                    DEFAULT 0
    }

    set ixnetwork_opt_args {
        -port_handle  REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
    }
    #not using lappend because that will break the existing list from opt_args
    set ixnetwork_opt_args "        \
            $ixnetwork_opt_args     \
            $opt_args"
   
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ospf_config $args $ixnetwork_man_args $ixnetwork_opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName:\
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_ospf_topology_route_config { args } {
    variable executeOnTclServer
	
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ospf_topology_route_config $args\}]

        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable new_ixnetwork_api
    
    variable  ospf_handles_array

    ::ixia::utrackerLog $procName $args

    keylset returnList status $::SUCCESS

    # Arguments
    set man_args {
        -mode                           CHOICES create modify delete enable disable
        -handle
    }

    set opt_args {
        -area_id                        IP
                                        DEFAULT 0.0.0.0
        -bfd_registration               CHOICES 0 1 
                                        DEFAULT 0
        -count                          RANGE 1-2000
        -dead_interval                  RANGE 0-65535
                                        DEFAULT 40
        -elem_handle
        -enable_advertise               CHOICES 0 1
                                        DEFAULT 1
        -enable_advertise_loopback      CHOICES 0 1
                                        DEFAULT 0
        
        -entry_point_address            IP
        -entry_point_prefix_length      RANGE 0-128
        -external_address_family        CHOICES unicast multicast
                                        DEFAULT unicast
        -external_ip_type               CHOICES ipv4 ipv6
                                        DEFAULT ipv6
        -external_number_of_prefix      RANGE 1-16000000
                                        DEFAULT 24
        -external_prefix_length         RANGE 0-128
        -external_prefix_metric         RANGE 0-16777215
                                        DEFAULT 1
        -external_prefix_start          IP
        -external_prefix_step           RANGE 0-2147483647
                                        DEFAULT 1
        -external_prefix_type           CHOICES 1 2
                                        DEFAULT 1
        -grid_col                       RANGE 0-10000
                                        DEFAULT 1
        -grid_connect                   DEFAULT 1 1
        -grid_disconnect
        -grid_link_type                 CHOICES broadcast ptop_numbered ptop_unnumbered
                                        DEFAULT ptop_numbered
        -grid_prefix_length             RANGE 0-128
        -grid_prefix_start              IP
        -grid_prefix_step               IP
        -grid_router_id                 IP
                                        DEFAULT 0.0.0.0
        -grid_router_id_step            IP
                                        DEFAULT 0.0.0.0
        -grid_row                       RANGE 0-10000
                                        DEFAULT 1
        -grid_te                        CHOICES 0 1
                                        DEFAULT 0
        -hello_interval                 RANGE 0-65535
                                        DEFAULT 10
        -interface_ip_address           IP
        -interface_ip_mask              IP
        -interface_ip_options
        -interface_metric               RANGE 0-65535
                                        DEFAULT 10
        -interface_mode                 CHOICES ospf_interface ospf_and_protocol_interface 
                                        DEFAULT ospf_and_protocol_interface
        -interface_mode2                CHOICES ospf_interface ospf_and_protocol_interface 
                                        DEFAULT ospf_and_protocol_interface
        -link_te                        CHOICES 0 1
        -link_te_metric                 RANGE 0-65535
                                        DEFAULT 10
        -link_te_max_bw                 DECIMAL
                                        DEFAULT 0
        -link_te_max_resv_bw            DECIMAL
                                        DEFAULT 0
        -link_te_unresv_bw_priority0    DECIMAL
                                        DEFAULT 0
        -link_te_unresv_bw_priority1    DECIMAL
                                        DEFAULT 0
        -link_te_unresv_bw_priority2    DECIMAL
                                        DEFAULT 0
        -link_te_unresv_bw_priority3    DECIMAL
                                        DEFAULT 0
        -link_te_unresv_bw_priority4    DECIMAL
                                        DEFAULT 0
        -link_te_unresv_bw_priority5    DECIMAL
                                        DEFAULT 0
        -link_te_unresv_bw_priority6    DECIMAL
                                        DEFAULT 0
        -link_te_unresv_bw_priority7    DECIMAL
                                        DEFAULT 0
        -link_type                      CHOICES external-capable ppp stub
        -neighbor_router_id             IPV4
        -neighbor_router_prefix_length  RANGE 0-32
                                        DEFAULT 32
        -net_ip                         IP
        -net_prefix_length              RANGE 1-128
        -net_prefix_options             DEFAULT 0
        -no_write                       FLAG
        -router_abr                     CHOICES 0 1
                                        DEFAULT 0
        -router_asbr                    CHOICES 0 1
                                        DEFAULT 0
        -router_id                      IP
                                        DEFAULT 0.0.0.0
        -router_te                      CHOICES 0 1
                                        DEFAULT 0
        -router_virtual_link_endpt      CHOICES 0 1
                                        DEFAULT 0
        -router_wcr                     CHOICES 0 1
                                        DEFAULT 0
        -summary_address_family         CHOICES unicast multicast
                                        DEFAULT unicast
        -summary_ip_type                CHOICES ipv4 ipv6
                                        DEFAULT ipv6
        -summary_number_of_prefix       RANGE 1-16000000
                                        DEFAULT 24
        -summary_prefix_length          RANGE 0-128
        -summary_prefix_metric          RANGE 0-16777215
        -summary_prefix_start           IP
                                        DEFAULT 1
        -summary_prefix_step            RANGE 0-2147483647
                                        DEFAULT 1
        -summary_route_type             CHOICES another_area same_area 
                                        DEFAULT another_area
        -type                           CHOICES router grid network summary_routes ext_routes
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ospf_topology_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName:\
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_ospf_control { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ospf_control $args\}]

        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable new_ixnetwork_api
    variable ospf_handles_array

    ::ixia::utrackerLog $procName $args

    # Arguments
    set man_args {
        -mode   CHOICES start stop restart
    }

    set opt_args {
        -port_handle REGEXP ^[0-9]+/[0-9]+/[0-9]+$
        -handle
    }

    if {[isUNIX] && [info exists ::ixTclSvrHandle]} {
        set retValueClicks [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock clicks}"]
        set retValueSeconds [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock seconds}"]
    } else {
        set retValueClicks [clock clicks]
        set retValueSeconds [clock seconds]
    }
    keylset returnList clicks [format "%u" $retValueClicks]
    keylset returnList seconds [format "%u" $retValueSeconds]

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ospf_control $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName:\
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_ospf_lsa_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ospf_lsa_config $args\}]

        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable new_ixnetwork_api
    variable  ospf_handles_array

    ::ixia::utrackerLog $procName $args

    keylset returnList status $::SUCCESS

    ## Hardcode the userLsaGroup.  Cisco's spec does not allow multiple
    ## userLsaGroups
    set userLsaGroupId  userLsaGroup1
    set routeRangeId    userRouteRange1

    # Arguments
    set man_args {
        -mode    CHOICES create modify delete reset
                 DEFAULT create
        -handle
    }

    set opt_args {
        -lsa_handle
        -adv_router_id                            IP 
                                                  DEFAULT 198.18.1.1
        -area_id                                  IP
                                                  DEFAULT 0.0.0.0
        -attached_router_id                       IP
                                                  DEFAULT 14.0.0.1
        -external_number_of_prefix                DEFAULT 16
        -external_prefix_start                    IP
                                                  DEFAULT 2000::1::1
        -external_prefix_length                   DEFAULT 16
        -external_prefix_step                     IP
                                                  DEFAULT 0.0.0.1
        -external_prefix_metric                   RANGE 0-16777215
                                                  DEFAULT 1
        -external_prefix_type                     CHOICES 1 2
                                                  DEFAULT 1
        -external_prefix_forward_addr             IP
                                                  DEFAULT 2001::1::1
        -external_route_tag                       IP
                                                  DEFAULT 17.0.0.1
        -external_metric_fbit                     CHOICES 1 0
                                                  DEFAULT 1
        -external_metric_tbit                     CHOICES 1 0
                                                  DEFAULT 1
        -external_metric_ebit                     CHOICES 1 0
                                                  DEFAULT 0
        -link_state_id                            IP 
                                                  DEFAULT 199.18.1.1
        -link_state_id_step                       IPV4
        -ls_type_function_code                    RANGE 0-8191
                                                  DEFAULT 0
        -lsa_group_mode                           CHOICES append create
                                                  DEFAULT append
        -net_prefix_length                        DEFAULT 16
        -net_attached_router                      CHOICES create delete reset
                                                  DEFAULT create
        -no_write                                 FLAG
        -opaque_enable_link_id                    CHOICES 0 1
                                                  DEFAULT 0
        -opaque_enable_link_local_ip_addr         CHOICES 0 1
                                                  DEFAULT 0
        -opaque_enable_link_max_bw                CHOICES 0 1
                                                  DEFAULT 0
        -opaque_enable_link_max_resv_bw           CHOICES 0 1
                                                  DEFAULT 0
        -opaque_enable_link_metric                CHOICES 0 1
                                                  DEFAULT 0
        -opaque_enable_link_remote_ip_addr        CHOICES 0 1
                                                  DEFAULT 0
        -opaque_enable_link_resource_class        CHOICES 0 1
                                                  DEFAULT 0
        -opaque_enable_link_type                  CHOICES 0 1
                                                  DEFAULT 0
        -opaque_enable_link_unresv_bw             CHOICES 0 1
                                                  DEFAULT 0
        -opaque_link_id                           IP 
                                                  DEFAULT 0.0.0.0
        -opaque_link_local_ip_addr                IP 
                                                  DEFAULT 0.0.0.0
        -opaque_link_max_bw                       DECIMAL 
                                                  DEFAULT 0
        -opaque_link_max_resv_bw                  DECIMAL 
                                                  DEFAULT 0
        -opaque_link_metric                       NUMERIC 
                                                  DEFAULT 0
        -opaque_link_remote_ip_addr               IP
                                                  DEFAULT 0.0.0.0
        -opaque_link_resource_class               HEX 
                                                  DEFAULT 0x00000000
        -opaque_link_type                         CHOICES ptop multiaccess
                                                  DEFAULT ptop
        -opaque_link_subtlvs                      IP
        -opaque_link_other_subtlvs                REGEXP ^(0x[0-9a-fA-F]+:[0-9]+:[0-9]+ )*0x[0-9a-fA-F]+:[0-9]+:[0-9]+$
        -opaque_link_unresv_bw_priority           DECIMAL 
                                                  DEFAULT 0
        -opaque_router_addr                       IP 
                                                  DEFAULT 0.0.0.0
        -opaque_tlv_type                          CHOICES link router 
                                                  DEFAULT router
        -options                                  RANGE 0-255
        -prefix_options                           RANGE 0-255
        -router_abr                               CHOICES 0 1
                                                  DEFAULT 0
        -router_asbr                              CHOICES 0 1
                                                  DEFAULT 0
        -router_virtual_link_endpt                CHOICES 0 1
                                                  DEFAULT 0
        -router_wildcard                          CHOICES 0 1
                                                  DEFAULT 0
        -router_link_mode                         CHOICES create modify delete
                                                  DEFAULT create
        -router_link_id                           IP
                                                  DEFAULT 12.0.0.1
        -router_link_data                         IP
                                                  DEFAULT 13.0.0.1
        -router_link_type                         CHOICES ptop transit stub virtual
                                                  DEFAULT ptop
        -router_link_metric                       RANGE 1-65535
                                                  DEFAULT 1
        -session_type                             CHOICES ospfv2 ospfv3
                                                  DEFAULT ospfv2
        -summary_number_of_prefix                 DEFAULT 16
        -summary_prefix_start                     IP
                                                  DEFAULT 15.0.0.1
        -summary_prefix_length                    DEFAULT 16
        -summary_prefix_step                      IP
                                                  DEFAULT 0.0.0.1
        -summary_prefix_metric                    RANGE 0-16777215
                                                  DEFAULT 1
        -type                                     CHOICES router network summary_pool
                                                  CHOICES asbr_summary ext_pool
                                                  CHOICES opaque_type_9 opaque_type_10 opaque_type_11
                                                  DEFAULT router
    }
    
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ospf_lsa_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName:\
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}

    return $returnList
}


proc ::ixia::emulation_ospf_info { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ospf_info $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    ::ixia::utrackerLog $procName $args
    
    set man_args {
        -mode          CHOICES aggregate_stats learned_info clear_stats
    }
    set opt_args {
        -port_handle   REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -handle
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ospf_info $args $man_args $opt_args]
        
    } else {
        set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
    }
    
    return $returnList
}
